#!/usr/local/bin/perl -w

use strict;
use warnings;


my $file_cdsDownLoaded = "cds_downloaded.fas";
my $file_aaDownLoaded  = "aa_downloaded.fas";

#####
open(INFILE,"$file_cdsDownLoaded") or die $!;
my @lines_cds = <INFILE>;
close INFILE;

open(INFILEAA,"$file_aaDownLoaded") or die $!;
my @lins_aa = <INFILEAA>;
close INFILEAA;


my ($ref_SpListArrayCDS, $ref_efasCDS) = &fasRead(\@lines_cds);
my ($ref_SpListArrayAA,  $ref_efasAA)  = &fasRead(\@lins_aa);
my $name_cds = $ref_SpListArrayCDS->[0];
chomp($name_cds);
my $seq_cds_1stStart = $$ref_efasCDS{$name_cds};



my $translation_1st_spaced = &translationANDspace($seq_cds_1stStart);

my $seq_cds_2ndStart = $seq_cds_1stStart;
$seq_cds_2ndStart =~ s/.//;
my $translation_2nd_spaced = &translationANDspace($seq_cds_2ndStart);

my $seq_cds_3rdStart = $seq_cds_1stStart;
$seq_cds_3rdStart =~ s/.//;
my $translation_3rd_spaced = &translationANDspace($seq_cds_3rdStart);




##### printout
my $AAname = $ref_SpListArrayAA->[0];
my $originalAA = $$ref_efasAA{$AAname };
my $spaceOriginalAA = $originalAA;
$spaceOriginalAA =~ s/(.)/$1  /g;


print "$AAname\n";
print "$originalAA\n";
print "$spaceOriginalAA\n";
print "\n";

print "$name_cds\n";
print "$seq_cds_1stStart\n";
print "$translation_1st_spaced\n";
print " "."$translation_2nd_spaced\n";
print "  "."$translation_3rd_spaced\n";
#print "$translation_1st\n";

exit;




####################
########  Subroutine
####################

sub translationANDspace {
  my($seq_cds) = @_;
  my $translation = "";
  for(my $i=0; $i < (length($seq_cds) - 2) ; $i += 3) {
    my $codon = substr($seq_cds, $i, 3);
    my $aa = DNA2AA_Nuc($codon, $i+1);
    $translation .= $aa."  ";
  }
  return($translation);
}

# ファスタ形式を取り込んでハッシュに格納する．
sub fasRead {
  my $name = "";
  my $seq = "";
  my $LineMilestone = 0;
  my @SpListArray = ( );   # name line の配列
  my %efas = ( );          # ファスタ形式をハッシュに入れる．


  my $ref_inFileCont = shift @_;
  
  foreach my $line (@$ref_inFileCont) {
    chomp($line);
    if ($line =~ /^>/) {
      if ($LineMilestone == 0) {    # 最初の name line
        $name = $line;
        $LineMilestone = 1;      
      } else {                      # 次からの name line
        push(@SpListArray,$name);   # name line を配列に格納する．
        $efas{$name} = $seq;        # ハッシュに fasta 形式を格納する．
  
        $name = '';
        $seq = '';
  
        $name = $line;    

      }
    } else {          # 配列を蓄積する．
      $seq .= $line;
    }
  }
  push(@SpListArray,$name);
  $efas{$name} = $seq;
    
  return (\@SpListArray,\%efas);
}




####### codon tables 

# DNA2AA_Nuc
#　核遺伝子の配列をアミノ酸に翻訳する．
# (transl_table=1)

sub DNA2AA_Nuc {

    my($tri, $site) = @_;
    
    $tri = uc $tri;   # すべて大文字に変換．
 
    my(%genetic_code) = (
    
    'TCA' => 'S',    # Serine
    'TCC' => 'S',    # Serine
    'TCG' => 'S',    # Serine
    'TCT' => 'S',    # Serine
    'TTC' => 'F',    # Phenylalanine
    'TTT' => 'F',    # Phenylalanine
    'TTA' => 'L',    # Leucine
    'TTG' => 'L',    # Leucine
    'TAC' => 'Y',    # Tyrosine
    'TAT' => 'Y',    # Tyrosine
    'TAA' => '*',    # Stop
    'TAG' => '*',    # Stop
    'TGC' => 'C',    # Cysteine
    'TGT' => 'C',    # Cysteine
    'TGA' => '*',    # Stop
    'TGG' => 'W',    # Tryptophan
    'CTA' => 'L',    # Leucine
    'CTC' => 'L',    # Leucine
    'CTG' => 'L',    # Leucine
    'CTT' => 'L',    # Leucine
    'CCA' => 'P',    # Proline
    'CCC' => 'P',    # Proline
    'CCG' => 'P',    # Proline
    'CCT' => 'P',    # Proline
    'CAC' => 'H',    # Histidine
    'CAT' => 'H',    # Histidine
    'CAA' => 'Q',    # Glutamine
    'CAG' => 'Q',    # Glutamine
    'CGA' => 'R',    # Arginine
    'CGC' => 'R',    # Arginine
    'CGG' => 'R',    # Arginine
    'CGT' => 'R',    # Arginine
    'ATA' => 'I',    # Isoleucine
    'ATC' => 'I',    # Isoleucine
    'ATT' => 'I',    # Isoleucine
    'ATG' => 'M',    # Methionine
    'ACA' => 'T',    # Threonine
    'ACC' => 'T',    # Threonine
    'ACG' => 'T',    # Threonine
    'ACT' => 'T',    # Threonine
    'AAC' => 'N',    # Asparagine
    'AAT' => 'N',    # Asparagine
    'AAA' => 'K',    # Lysine
    'AAG' => 'K',    # Lysine
    'AGC' => 'S',    # Serine
    'AGT' => 'S',    # Serine
    'AGA' => 'R',    # Arginine
    'AGG' => 'R',    # Arginine
    'GTA' => 'V',    # Valine
    'GTC' => 'V',    # Valine
    'GTG' => 'V',    # Valine
    'GTT' => 'V',    # Valine
    'GCA' => 'A',    # Alanine
    'GCC' => 'A',    # Alanine
    'GCG' => 'A',    # Alanine
    'GCT' => 'A',    # Alanine
    'GAC' => 'D',    # Aspartic Acid
    'GAT' => 'D',    # Aspartic Acid
    'GAA' => 'E',    # Glutamic Acid
    'GAG' => 'E',    # Glutamic Acid
    'GGA' => 'G',    # Glycine
    'GGC' => 'G',    # Glycine
    'GGG' => 'G',    # Glycine
    'GGT' => 'G',    # Glycine
    '---' => '-',    # gap
    'NNN' => 'N',    # gap
    );
    

    if(exists $genetic_code{$tri}) {
      return $genetic_code{$tri};
    }else{
      return "X  ";
    }
}

